@@ -417,6 +417,10 @@ Examples:
 						break
 					}
 
+					if len(record.Seq.Seq) == 0 {
+						continue
+					}
+
 					if checkAlphabet {
 						if fastxReader.Alphabet() == seq.Unlimit || fastxReader.Alphabet() == seq.Protein {
 							onlyPositiveStrand = true
@@ -545,6 +549,10 @@ Examples:
 					break
 				}
 
+				if len(record.Seq.Seq) == 0 {
+					continue
+				}
+
 				if checkAlphabet {
 					if fastxReader.Alphabet() == seq.Unlimit || fastxReader.Alphabet() == seq.Protein {
 						onlyPositiveStrand = true
