@@ -49,8 +49,7 @@ occupation and fast speed.
 The prefix of output files:
   1. For stdin: stdin
   2. Others: same to the input file
-  3. Set via the option: -o/--out-file, e.g., outputting xxx.part_001.fasta:
-       cat ../tests/hairpin.fa | ./seqkit split2 -p 2 -O test -o xxx
+  3. Set via the options: --by-length-prefix, --by-part-prefix, or --by-size-prefix
 
 The extension of output files:
   1. For stdin: .fast[aq]
@@ -99,8 +98,17 @@ The extension of output files:
 
 		outdir := getFlagString(cmd, "out-dir")
 		force := getFlagBool(cmd, "force")
+
 		extension := getFlagString(cmd, "extension")
 
+		prefixBySize := getFlagString(cmd, "by-size-prefix")
+		prefixByPart := getFlagString(cmd, "by-part-prefix")
+		prefixByLength := getFlagString(cmd, "by-length-prefix")
+
+		prefixBySizeSet := cmd.Flags().Lookup("by-size-prefix").Changed
+		prefixByPartSet := cmd.Flags().Lookup("by-part-prefix").Changed
+		prefixByLengthSet := cmd.Flags().Lookup("by-length-prefix").Changed
+
 		if size == 0 && parts == 0 && length == 0 {
 			checkError(fmt.Errorf(`one of flags should be given: -s/-p/-l. type "seqkit split2 -h" for help`))
 		}
@@ -218,12 +226,14 @@ The extension of output files:
 				var counts []int
 				var outfiles []string
 
+				// by size or by length
+				var outfhPre *xopen.Writer
+				var prefix string
+				var outfilePre string
+
 				var flag bool
 
 				if bySize || byLength { // by size or by length
-					outfhs = make([]*xopen.Writer, 0, 10)
-					counts = make([]int, 0, 10)
-					outfiles = make([]string, 0, 10)
 				} else if byParts { // by part
 					outfhs = make([]*xopen.Writer, 0, parts)
 					counts = make([]int, 0, parts)
@@ -237,6 +247,7 @@ The extension of output files:
 				i := 0 // nth part
 				j := 0
 				var n int64 // length sum
+				once := true
 				for {
 					record, err = fastxReader.Read()
 					if err != nil {
@@ -247,141 +258,170 @@ The extension of output files:
 						break
 					}
 
-					if fastxReader.IsFastq {
-						config.LineWidth = 0
-						fastx.ForcelyOutputFastq = true
+					if once {
+						if fastxReader.IsFastq {
+							config.LineWidth = 0
+							fastx.ForcelyOutputFastq = true
+						}
+
+						if renameFileExt && isstdin {
+							if len(record.Seq.Qual) > 0 {
+								fileExt = suffixFQ + extension
+							} else {
+								fileExt = suffixFA + extension
+							}
+							renameFileExt = false
+						}
+						once = false
 					}
 
-					if renameFileExt && isstdin {
-						if len(record.Seq.Qual) > 0 {
-							fileExt = suffixFQ + extension
-						} else {
-							fileExt = suffixFA + extension
-						}
-						renameFileExt = false
-					}
 					n += int64(len(record.Seq.Seq))
 
 					if bySize {
 						if j == size {
-							outfhs[i].Close()
+							outfhPre.Close()
 							if !quiet {
-								log.Infof("write %d sequences to file: %s\n", counts[i], outfiles[i])
+								log.Infof("write %d sequences to file: %s\n", j, outfilePre)
 							}
-							outfhs[i] = nil
 
 							i++
-							var outfh2 *xopen.Writer
-							outfile := filepath.Join(outdir, fmt.Sprintf("%s.part_%03d%s", filepath.Base(fileName), i+1, fileExt))
-							outfh2, err = xopen.Wopen(outfile)
-							checkError(err)
 
-							outfhs = append(outfhs, outfh2)
-							counts = append(counts, 0)
-							outfiles = append(outfiles, outfile)
+							if prefixBySizeSet {
+								prefix = prefixBySize
+							} else {
+								prefix = fmt.Sprintf("%s.part_", filepath.Base(fileName))
+							}
+							outfilePre = filepath.Join(outdir, fmt.Sprintf("%s%03d%s", prefix, i+1, fileExt))
+							outfhPre, err = xopen.Wopen(outfilePre)
+							checkError(err)
 
 							j = 0
 						}
 					} else if byLength {
 						flag = false
-						if n >= length {
-							if len(outfhs) == 0 { // order is different
-								var outfh2 *xopen.Writer
-								outfile := filepath.Join(outdir, fmt.Sprintf("%s.part_%03d%s", filepath.Base(fileName), i+1, fileExt))
-								outfh2, err = xopen.Wopen(outfile)
-								checkError(err)
 
-								outfhs = append(outfhs, outfh2)
-								counts = append(counts, 0)
-								outfiles = append(outfiles, outfile)
-
-								record.FormatToWriter(outfhs[i], config.LineWidth)
-								counts[i]++
-
-								outfhs[i].Close()
-								if !quiet {
-									log.Infof("write %d sequences to file: %s\n", counts[i], outfiles[i])
-								}
-								outfhs[i] = nil
-								i++
-
-								n = 0
+						if outfhPre == nil { // first record
+							var outfh2 *xopen.Writer
+							if prefixByLengthSet {
+								prefix = prefixByLength
 							} else {
-								record.FormatToWriter(outfhs[i], config.LineWidth)
-								counts[i]++
-
-								outfhs[i].Close()
-								if !quiet {
-									log.Infof("write %d sequences to file: %s\n", counts[i], outfiles[i])
-								}
-								outfhs[i] = nil
-								i++
-
-								var outfh2 *xopen.Writer
-								outfile := filepath.Join(outdir, fmt.Sprintf("%s.part_%03d%s", filepath.Base(fileName), i+1, fileExt))
-								outfh2, err = xopen.Wopen(outfile)
-								checkError(err)
-
-								outfhs = append(outfhs, outfh2)
-								counts = append(counts, 0)
-								outfiles = append(outfiles, outfile)
-
-								n = 0
+								prefix = fmt.Sprintf("%s.part_", filepath.Base(fileName))
 							}
+							outfile := filepath.Join(outdir, fmt.Sprintf("%s%03d%s", prefix, i+1, fileExt))
+							outfh2, err = xopen.Wopen(outfile)
+							checkError(err)
+
+							outfhPre = outfh2
+							outfilePre = outfile
+
+							j = 0
+						}
+
+						if n >= length {
+							record.FormatToWriter(outfhPre, config.LineWidth)
+							j++
+
+							outfhPre.Close()
+							if !quiet {
+								log.Infof("write %d sequences to file: %s\n", j, outfilePre)
+							}
+							i++
+
+							var outfh2 *xopen.Writer
+
+							if prefixByLengthSet {
+								prefix = prefixByLength
+							} else {
+								prefix = fmt.Sprintf("%s.part_", filepath.Base(fileName))
+							}
+							outfile := filepath.Join(outdir, fmt.Sprintf("%s%03d%s", prefix, i+1, fileExt))
+							outfh2, err = xopen.Wopen(outfile)
+							checkError(err)
+
+							outfhPre = outfh2
+							outfilePre = outfile
+
+							j = 0
+							n = 0
 							flag = false
 						} else { // write this record later
 							flag = true
 						}
 					}
 
-					if i+1 > len(outfhs) || outfhs[i] == nil {
-						var outfh2 *xopen.Writer
-						outfile := filepath.Join(outdir, fmt.Sprintf("%s.part_%03d%s", filepath.Base(fileName), i+1, fileExt))
-						outfh2, err = xopen.Wopen(outfile)
-						checkError(err)
+					if bySize {
+						// first record, for bySize
+						if outfhPre == nil {
+							if prefixBySizeSet {
+								prefix = prefixBySize
+							} else {
+								prefix = fmt.Sprintf("%s.part_", filepath.Base(fileName))
+							}
+							outfilePre = filepath.Join(outdir, fmt.Sprintf("%s%03d%s", prefix, i+1, fileExt))
+							outfhPre, err = xopen.Wopen(outfilePre)
+							checkError(err)
 
-						outfhs = append(outfhs, outfh2)
-						counts = append(counts, 0)
-						outfiles = append(outfiles, outfile)
-					}
+							j = 0
+						}
 
-					if byLength {
+						record.FormatToWriter(outfhPre, config.LineWidth)
+
+						j++ // increase size
+					} else if byLength {
 						if flag {
-							record.FormatToWriter(outfhs[i], config.LineWidth)
-							counts[i]++
+							record.FormatToWriter(outfhPre, config.LineWidth)
+
+							j++
 						}
 					} else {
+						// first record, for byParts
+						if i+1 > len(outfhs) {
+							var outfh2 *xopen.Writer
+							if prefixByPartSet {
+								prefix = prefixByPart
+							} else {
+								prefix = fmt.Sprintf("%s.part_", filepath.Base(fileName))
+							}
+							outfile := filepath.Join(outdir, fmt.Sprintf("%s%03d%s", prefix, i+1, fileExt))
+							outfh2, err = xopen.Wopen(outfile)
+							checkError(err)
+
+							outfhs = append(outfhs, outfh2)
+							counts = append(counts, 0)
+							outfiles = append(outfiles, outfile)
+						}
+
 						record.FormatToWriter(outfhs[i], config.LineWidth)
 						counts[i]++
-					}
 
-					if bySize {
-						j++ // increase size
-					} else if byParts {
 						i++
 						if i == parts { // reset index
 							i = 0
 						}
-					} else {
-
 					}
+
 				}
 
-				// for by-size: only log last part,
-				// for by-parts: log all parts.
-				for i, outfh := range outfhs {
-					if outfh == nil {
-						continue
-					}
-					outfh.Close()
+				if byParts {
+					for i, outfh := range outfhs {
+						outfh.Close()
 
-					if !quiet {
-						if counts[i] == 0 {
-							os.Remove(outfiles[i])
-						} else {
-							log.Infof("write %d sequences to file: %s\n", counts[i], outfiles[i])
+						if !quiet {
+							if counts[i] == 0 {
+
+							} else {
+								log.Infof("write %d sequences to file: %s\n", counts[i], outfiles[i])
+							}
 						}
 					}
+				} else {
+					outfhPre.Close()
+
+					if j == 0 {
+						os.Remove(outfilePre)
+					} else {
+						log.Infof("write %d sequences to file: %s\n", j, outfilePre)
+					}
 				}
 
 			}(file)
@@ -402,5 +442,9 @@ func init() {
 	split2Cmd.Flags().StringP("out-dir", "O", "", "output directory (default value is $infile.split)")
 	split2Cmd.Flags().BoolP("force", "f", false, "overwrite output directory")
 
+	split2Cmd.Flags().StringP("by-size-prefix", "", "", "file prefix for --by-size")
+	split2Cmd.Flags().StringP("by-part-prefix", "", "", "file prefix for --by-part")
+	split2Cmd.Flags().StringP("by-length-prefix", "", "", "file prefix for --by-length")
+
 	split2Cmd.Flags().StringP("extension", "e", "", `set output file extension, e.g., ".gz", ".xz", or ".zst"`)
 }
