@@ -33,7 +33,6 @@ import (
 	"github.com/bsipos/thist"
 	"github.com/shenwei356/bio/seq"
 	"github.com/shenwei356/bio/seqio/fastx"
-	"github.com/shenwei356/util/byteutil"
 	"github.com/shenwei356/xopen"
 	"github.com/spf13/cobra"
 )
@@ -168,7 +167,7 @@ var watchCmd = &cobra.Command{
 		var head []byte
 		var sequence *seq.Seq
 		var text []byte
-		var b *bytes.Buffer
+		var buffer *bytes.Buffer
 		var record *fastx.Record
 		var fastxReader *fastx.Reader
 		var count int
@@ -231,47 +230,28 @@ var watchCmd = &cobra.Command{
 				head = record.Name
 
 				if isFastq {
-					outfh.WriteString("@")
+					outfh.Write(_mark_fastq)
 					outfh.Write(head)
-					outfh.WriteString("\n")
+					outfh.Write(_mark_newline)
 				} else {
-					outfh.WriteString(">")
+					outfh.Write(_mark_fasta)
 					outfh.Write(head)
-					outfh.WriteString("\n")
+					outfh.Write(_mark_newline)
 				}
 
 				sequence = record.Seq
 
-				if len(sequence.Seq) <= pageSize {
-					outfh.Write(byteutil.WrapByteSlice(sequence.Seq, config.LineWidth))
-				} else {
-					if bufferedByteSliceWrapper == nil {
-						bufferedByteSliceWrapper = byteutil.NewBufferedByteSliceWrapper2(1, len(sequence.Seq), config.LineWidth)
-					}
-					text, b = bufferedByteSliceWrapper.Wrap(sequence.Seq, config.LineWidth)
-					outfh.Write(text)
-					outfh.Flush()
-					bufferedByteSliceWrapper.Recycle(b)
-				}
+				text, buffer = wrapByteSlice(sequence.Seq, config.LineWidth, buffer)
+				outfh.Write(text)
 
-				outfh.WriteString("\n")
+				outfh.Write(_mark_newline)
 
 				if printQual {
-					outfh.WriteString("+\n")
+					outfh.Write(_mark_plus_newline)
 
-					if len(sequence.Qual) <= pageSize {
-						outfh.Write(byteutil.WrapByteSlice(sequence.Qual, config.LineWidth))
-					} else {
-						if bufferedByteSliceWrapper == nil {
-							bufferedByteSliceWrapper = byteutil.NewBufferedByteSliceWrapper2(1, len(sequence.Qual), config.LineWidth)
-						}
-						text, b = bufferedByteSliceWrapper.Wrap(sequence.Qual, config.LineWidth)
-						outfh.Write(text)
-						outfh.Flush()
-						bufferedByteSliceWrapper.Recycle(b)
-					}
-
-					outfh.WriteString("\n")
+					text, buffer = wrapByteSlice(sequence.Qual, config.LineWidth, buffer)
+					outfh.Write(text)
+					outfh.Write(_mark_newline)
 				}
 
 			} // record
